import { logger, selectImplementations, } from '@webprovisions/platform';
import { queryLinkTargets, queryPreRenderedTargets } from './target-queries';
import TriggerElementMonitor from './trigger-element-monitor';
import distinct from './distinct';
export function attach(widget) {
    var triggerElements = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        triggerElements[_i - 1] = arguments[_i];
    }
    widget.implementation.events.subscribe('widget:create-instance', function (e, data) {
        /**
         * Listen through the implementation instead of the widget directly, else
         * event listeners will be removed if the widget is deactivated and re-activated.
         */
        if (e.target === widget) {
            var bootstrapping = widget.container.get('bootstrapping');
            if (!bootstrapping.monitor) {
                // create and register if not available
                bootstrapping.monitor = new TriggerElementMonitor(widget);
            }
            bootstrapping.monitor.watch(triggerElements);
        }
    });
    return widget.load();
}
export function scan(implementation, rootElement) {
    if (rootElement === void 0) { rootElement = document; }
    // scan 'rootElement' for trigger elements matching the bindings
    var bindings = implementation.bindings;
    var triggerElementMap = [].concat(queryLinkTargets(rootElement, bindings), queryPreRenderedTargets(rootElement, bindings));
    // iterate each unique widget name and attach it to its detected trigger elements
    var uniqueWidgetNames = distinct(triggerElementMap.map(function (_a) {
        var widgetName = _a.widgetName;
        return widgetName;
    }));
    return Promise.all(uniqueWidgetNames.map(function (widgetName) {
        var widget = implementation.widgets.get(widgetName);
        if (!widget) {
            if (process.env.NODE_ENV !== 'production') {
                // tslint:disable-next-line max-line-length
                logger(widget).warn("A trigger element for \"" + widgetName + "\" was found but no widget with that name is available on the implementation (\"" + implementation.name + "\"). Make sure this is not a mistake.");
            }
            return;
        }
        var triggerElements = triggerElementMap
            .filter(function (item) { return item.widgetName === widgetName; })
            .map(function (item) { return item.element; });
        return attach.apply(void 0, [widget].concat(triggerElements));
    }));
}
function domContentLoaded() {
    return new Promise(function (resolve) {
        if (document.readyState === 'loading') {
            document.addEventListener('DOMContentLoaded', function () { return resolve(); });
        }
        else {
            resolve();
        }
    });
}
var extendImplementation = function (implementation, bindings) {
    var bootstrappedImplementation = implementation;
    bootstrappedImplementation.scan = function (rootElement) {
        return scan(bootstrappedImplementation, rootElement);
    };
    bootstrappedImplementation.bindings = bindings;
    bootstrappedImplementation.attach = function (widget) {
        var triggerElements = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            triggerElements[_i - 1] = arguments[_i];
        }
        return attach.apply(void 0, [widget].concat(triggerElements));
    };
    var environment = bootstrappedImplementation.environment;
    if (typeof environment.configure === 'undefined') {
        environment.configure = function (selectorOrConfigFn, configFn) {
            var selector;
            var fn;
            if (configFn) {
                selector = selectorOrConfigFn;
                fn = configFn;
            }
            else {
                selector = '*';
                fn = selectorOrConfigFn;
            }
            environment.implementations.query(selector)
                .forEach(function (implementation) { return implementation.configure(fn); });
            environment.events.subscribe('implementation:created', function (event, data) {
                var implementation = data.implementation;
                if (selectImplementations(selector)(implementation)) {
                    implementation.configure(fn);
                }
            });
        };
    }
    return bootstrappedImplementation;
};
/**
 * Bootstraps an implementation by scanning the document, or the specified root element, for
 * trigger elements, and enables the default bootstrapping capabilities in supported widgets.
 * @param implementation Implementation to bootstrap.
 * @param createConfigurationHandlers Function for supplying default configuration handlers.
 * @param rootElementResolver Function for resolving the root element.
 */
export default function bootstrapImplementation(implementation, bindings, createConfigurationHandlers, rootElementResolver) {
    if (createConfigurationHandlers === void 0) { createConfigurationHandlers = function () { return []; }; }
    var bootstrapping = { ready: false, monitor: null };
    implementation.container.register('bootstrapping', bootstrapping);
    var bootstrappedImplementation = extendImplementation(implementation, bindings);
    return domContentLoaded()
        .then(function () {
        implementation.configure(function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            createConfigurationHandlers()
                .forEach(function (configureHandler) { return configureHandler.apply(void 0, args); });
        });
    })
        .then(function () { return scan(bootstrappedImplementation, rootElementResolver()); })
        .then(function () {
        bootstrapping.ready = true;
        implementation.events.dispatch('bootstrap:ready', {});
        return bootstrappedImplementation;
    });
}
//# sourceMappingURL=bootstrap-implementation.js.map